from fastapi import APIRouter, Depends, HTTPException
from pydantic import BaseModel, Field
from starlette import status

from api import get_conn
from util.auth import check_auth
from util.util import send_email
from model.settings import settings
from model.wapi.cntl import Mgr, APIToken
from util.wapi_util import execute_wapi_function

router = APIRouter(
    prefix="/api/ip_contact",
    tags=["ip_contact"]
)
js_mods = ['ip_contact']


class IPContactRequest(BaseModel):
    cidr: str = Field()
    message: str = Field()


class IPContactResponse(BaseModel):
    success: bool = Field(default=True)


@router.post('')
async def send_ip_contact(contact_request: IPContactRequest,
                          token: APIToken = Depends(check_auth),
                          conn=Depends(get_conn)
                          ) -> IPContactResponse:
    user = Mgr.check_token(conn, token)

    if not contact_request.message or len(contact_request.message) == 0:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail={
                'en': 'Message is required!',
                'de': 'Es muss eine Nachricht angegeben werden!'
            }
        )

    r = execute_wapi_function(conn, [
        {'idx': 'subnet_list', 'name': 'nd.ip_subnet.list', 'old': {
            'cidr': contact_request.cidr,
            'cidr_operator': 'smallest_cts'
        }},
        {'idx': 'bcd_list', 'name': 'nd.bcd.list', 'inner_join_ref': {
            'subnet_list': 'default'
        }},
        {'idx': 'bcd2group_list', 'name': 'nd.bcd2group.list', 'inner_join_ref': {
            'bcd_list': 'default'
        }},
        {'idx': 'group_list', 'name': 'cntl.group.list', 'old': {
            'is_sub': False
        }, 'inner_join_ref': {
            'bcd2group_list': 'default'
        }},
        {'idx': 'group2mgr_list', 'name': 'cntl.mgr2group.list', 'inner_join_ref': {
            'group_list': 'default'
        }},
        {'idx': 'mgr_list', 'name': 'cntl.mgr.list', 'old': {
            'is_sub': False,
            'is_svc': False,
        }, 'inner_join_ref': {
            'group2mgr_list': 'default'
        }}
    ], superuser=True)

    if len(r['subnet_list']) == 0:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail={
                'en': 'Matching subnet not found!',
                'de': 'Kein passendes Subnetz gefunden!'
            }
        )

    bcd = r['subnet_list'][0]['bcd']
    emails = [m['email'] for m in r['mgr_list'] if m['email'] is not None]

    if len(emails) == 0:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail={
                'en': 'No managers found for requested subnet!',
                'de': 'Kein Betreuer für das angefragte Subnetz gefunden!'
            }
        )

    if settings.ip_contact_override_mail_receiver is not None:
        emails = settings.ip_contact_override_mail_receiver.split(',')

    if settings.ip_contact_send_mail:
        send_email(
            to=emails,
            sender=settings.ip_contact_email_sender,
            reply_to=user.email,
            subject='NETVS: IP-Kontakt zu {bcd} ({ip})'.format(bcd=bcd, ip=contact_request.cidr),
            body="""Diese Nachricht stammt aus dem IP-Kontakt-Formular des Netzdiensteportals des SCCs (NETVS).
Durch Antworten auf diese E-Mail antworten Sie direkt dem Absender des Formulars.

Angefragtes Netz/IP: {bcd} {ip}
Nachricht:

{msg}
            """.format(bcd=bcd, ip=contact_request.cidr, msg=contact_request.message),
            body_type='plain',
        )

        send_email(
            to=user.email,
            sender=settings.ip_contact_email_sender,
            subject='NETVS: Bestätigung IP-Kontakt zu {ip}'.format(ip=contact_request.cidr),
            body="""Ihre Anfrage an die Adressbetreuer von {ip} wurde erfolgreich zugestellt.
Ihre Nachricht:

{msg}
            """.format(ip=contact_request.cidr, msg=contact_request.message),
            body_type='plain',
        )

    return IPContactResponse(success=True)
